# Module for reading shell-compatible config files.

class SyntaxError(Exception):
    pass

def read_file(path):
    config = {}
    config_file = file(path)
    line_no = 0
    try:
        for line in config_file:
            line_no += 1
            # Find first non-space.
            col = 0
            while col != len(line) and line[col].isspace():
                col += 1
            if col == len(line) or line[col] == '#':
                # This is a blank or comment line.
                continue
            elif line[col].isalpha() or line[col] == '_':
            # Find end of name.
                name_begin = col
                while (col != len(line)
                       and (line[col].isalpha() or line[col] == '_')):
                    col += 1
                name = line[name_begin:col]
                if line[col] != '=':
                    raise SyntaxError(path, line,
                                      'no = found after variable name')
                col += 1
                value = ''
                while col != len(line) and not line[col].isspace():
                    if line[col] in '\'"':
                        quote = line[col]
                        col += 1
                    else:
                        quote = None
                    while (col != len(line)
                           and not (quote and line[col] == quote)
                           and not (not quote and (line[col].isspace()
                                                   or line[col] in '\'"'))):
                        ch = line[col]
                        col += 1
                        if quote != "'":
                            if ch == '$':
                                raise SyntaxError(path, line,
                                                  '$-expansion not supported')
                            elif ch == '\\':
                                # Check whether it's a valid escape.
                                ch = line[col]
                                if ch in '$\'"\\' or (quote is None
                                                      and ch == ' '):
                                    col += 1
                                else:
                                    raise SyntaxError(
                                        path, line,
                                        'invalid escape sequence')
                        value += ch
                    if quote and col != len(line):
                        col += 1
                while (col != len(line)
                       and line[col].isspace() and line[col] != '\n'):
                    col += 1
                if col == len(line) or line[col] != '\n':
                    raise SyntaxError(path, line,
                                      'unexpected character after assignment')
                config[name] = value
            else:
                raise SyntaxError(path, line,
                                  'invalid variable name character')
    finally:
        config_file.close()
    return config
